//
//  LLConcurrentOperation.h
//  LLCoreConcurrency
//
//  Created by Damien DeVille on 4/16/14.
//  Copyright (c) 2014 Damien DeVille. All rights reserved.
//

#import <Foundation/Foundation.h>

/*!
	\brief
	This class represents an abstract concurrent operation
 */
@interface LLConcurrentOperation : NSOperation

/*!
	\brief
	Designated initializer
 */
- (id)init;

/*!
	\brief
	Support creating a concurrent operation with a given work queue
 */
- (id)initWithWorkQueue:(NSOperationQueue *)workQueue;

/*!
	\brief
	Override this in the subclass to provide work operations to be executed
 */
- (void)setupOperations:(void (^)(NSOperation *operation))addWorkOperation;

/*!
	\brief
	Override this in the subclass to do cancellation work. This will be invoked from the `cancel` method in a concurrency-safe manner.
 */
- (void)cancelWork;

/*!
	\brief
	Cleanup is called when the operation goes isFinished
 */
- (void)cleanup;

@end